<?php

class NewsletterElasticEmail extends NewsletterMailerAddon {

    /**
     * @var NewsletterElasticEmail
     */
    static $instance;

    /* @var NewsletterLogger */
    var $logger;

    function __construct($version) {
        self::$instance = $this;
        parent::__construct('elasticemail', $version);
    }

    function get_description() {
        return 'ElasticEmail Addon';
    }

    function get_mailer() {
        static $mailer = null;
        if (!$mailer) {
            $mailer = new NewsletterElasticEmailMailer($this->name, $this->options);
        }
        return $mailer;
    }

    function init() {
        parent::init();
        add_action('newsletter_elasticemail_bounce', array($this, 'bounce'));

        // Earlier than unsubscription module
        if ($this->enabled) {
            add_action('newsletter_replace', array($this, 'hook_newsletter_replace'), 5, 3);            
        }
        
        if (is_admin()) {
            if (current_user_can('administrator')) {
                add_action('admin_menu', array($this, 'hook_admin_menu'), 100);
                add_filter('newsletter_menu_settings', array($this, 'hook_newsletter_menu_settings'));
            }
            if (!defined('DOING_CRON') || !DOING_CRON) {
                if (wp_get_schedule('newsletter_elasticemail_bounce') === false) {
                    wp_schedule_event(time() + 60, 'daily', 'newsletter_elasticemail_bounce');
                }
            }
        }

        add_action('newsletter_user_deleted', array($this, 'hook_newsletter_user_deleted'));
    }
    
    function hook_newsletter_replace($text, $user, $email) {;
        $text = str_replace('{unsubscription_url}', '{unsubscribe:{unsubscription_url}}', $text);
        $text = str_replace('{unsubscription_confirm_url}', '{unsubscribe:{unsubscription_confirm_url}}', $text);
        return $text;
    }

    function hook_newsletter_menu_settings($entries) {
        $entries[] = array('label' => '<i class="fa fa-envelope"></i> Elastic Email', 'url' => '?page=newsletter_elasticemail_index', 'description' => 'Send emails with ElasticEmail');
        return $entries;
    }

    function hook_admin_menu() {
        add_submenu_page('newsletter_main_index', 'Elastic Email', '<span class="tnp-side-menu">Elastic Email</span>', 'manage_options', 'newsletter_elasticemail_index', array($this, 'menu_page_index'));
    }

    function menu_page_index() {
        global $wpdb;
        require dirname(__FILE__) . '/index.php';
    }

    /**
     * 
     * @param TNP_User $user
     * @return type
     */
    function hook_newsletter_user_deleted($user) {
        $logger = $this->get_logger();
        $this->logger->debug('Deleting user ' . $user->email);
        $data['apikey'] = $this->options['api_key'];
        $data['emails'] = $user->email;
        $response = wp_remote_post("https://api.elasticemail.com/v2//contact/delete", array('body' => $data));
        if (is_wp_error($response)) {
            $this->logger->error($response);
            return;
        }
        $result = json_decode(wp_remote_retrieve_body($response));
        // Returns ok even if the contact is not there, and that's good
        if ($result->success === false) {
            $this->logger->error($result);
            return;
        }
        $this->logger->debug($result);
    }

    /**
     * 
     * @global wpdb $wpdb
     */
    var $bounce_result;
    var $bounce_emails = array();
    var $unsubscribed_emails = array();

    function bounce() {
        global $wpdb, $newsletter;
        $logger = $this->get_logger();

        $data['apikey'] = $this->options['api_key'];
        $data['statuses'] = 'Bounced,Unsubscribed';
        $data['search'] = '';
        $data['limit'] = 1000;

        $response = wp_remote_post("https://api.elasticemail.com/v2/contact/loadblocked", array('body' => $data));

        if (is_wp_error($response)) {
            /* @var $response WP_Error */
            $this->bounce_result = $response->get_error_message();
            $logger->error($response);
            return false;
        }

        $http_code = wp_remote_retrieve_response_code($response);
        $http_body = wp_remote_retrieve_body($response);
        if ($http_code != 200) {
            $this->bounce_result = 'HTTP Error ' . $http_code . '<br>' . $http_body;
            $logger->error('HTTP Error ' . $http_code);
            $logger->error($http_body);
            return false;
        }

        $json = json_decode($http_body);

        if (!$json) {
            $this->bounce_result = 'Unable to decode the answer';
            $logger->error('Unable to decode the answer');
            $logger->error($http_body);
            return false;
        }

        if ($json->success === false) {
            $this->bounce_result = $json->error;
            $logger->error($json->error);
            return false;
        }

        $logger->debug($json);

        foreach ($json->data as $item) {
            $status = 'B';
            if ($item->status == 2) {
                $status = 'U';
                $this->unsubscribed_emails[] = $item->email;
            } else {
                $this->bounce_emails[] = $item->email;
            }

            $email = strtolower($item->email);
            $wpdb->query($wpdb->prepare("update " . NEWSLETTER_USERS_TABLE . " set status=%s where email=%s limit 1", $status, $email));
        }

        $this->save_last_run(time());
    }

}

class NewsletterElasticEmailMailer extends NewsletterMailer {

    function __construct($name, $options = array()) {
        parent::__construct($name, $options);
        if (!empty($this->options['turbo'])) {
            $this->batch_size = (int) $this->options['turbo'];
        }
    }

    /**
     * 
     * @param TNP_Mailer_Message $message
     * @return array
     */
    function build_data($message) {
        $logger = $this->get_logger();
        $newsletter = Newsletter::instance();

        $data = array();

        $data['subject'] = $message->subject;
        $data['from'] = $newsletter->options['sender_email'];
        $data['fromName'] = $newsletter->options['sender_name'];

        $data['replyTo'] = $newsletter->options['reply_to'];
        $data['to'] = $message->to;
        
        $i=0;
        foreach($message->headers as $key=>$value) {
            $data['headers_' . $i] = $key . ': ' . $value;
        }
        
        $data['bodyHtml'] = $message->body;
        if (empty($message->body) || empty($this->options['disable_text_part'])) {
            $data['bodyText'] = $message->body_text;
        }
            

        $data['apikey'] = $this->options['api_key'];

        return $data;
    }

    /**
     * 
     * @param TNP_Mailer_Message $message
     * @return type
     */
    function build_curl($message) {
        $curl = curl_init('https://api.elasticemail.com/v2/email/send');
        curl_setopt($curl, CURLOPT_POST, true);
        $data = $this->build_data($message);
        curl_setopt($curl, CURLOPT_POSTFIELDS, $data);
        curl_setopt($curl, CURLOPT_CONNECTTIMEOUT, 10);
        curl_setopt($curl, CURLOPT_TIMEOUT, 60);
        curl_setopt($curl, CURLOPT_SSL_VERIFYPEER, false);
        // Tell curl not to return headers, but do return the response
        curl_setopt($curl, CURLOPT_HEADER, false);
        curl_setopt($curl, CURLOPT_RETURNTRANSFER, true);
        return $curl;
    }

    /**
     * 
     * @param TNP_Mailer_Message $message
     * @return \WP_Error|boolean
     */
    function send($message) {
        $logger = $this->get_logger();
        $newsletter = Newsletter::instance();

        $curl = $this->build_curl($message);

        $response = curl_exec($curl);

        if (curl_errno($curl) != 0) {
            curl_close($curl);
            $logger->error('cURL error: ' . curl_errno($curl) . ' - ' . curl_error($curl));
            $message->error = curl_error($curl);
            curl_close($curl);
            return new WP_Error(self::ERROR_GENERIC, curl_errno($curl) . ' - ' . curl_error($curl));
        }

        $code = curl_getinfo($curl, CURLINFO_HTTP_CODE);

       if ($code < 200 || $code > 299) {
            $logger->error('Http error ' . $code);
            $message->error = curl_error('Http error ' . $code);
            curl_close($curl);
            return new WP_Error(self::ERROR_GENERIC, 'Http error ' . $code);
        }

        $jsonResult = json_decode($response);

        if ($jsonResult === false) {
            curl_close($curl);
            $message->error = 'Unable to decode the response: ' . $response;
            return new WP_Error(self::ERROR_GENERIC, 'Unable to decode the response: ' . $response);
        }

        if ($jsonResult->success === false) {
            curl_close($curl);
            $message->error = $jsonResult->error;
            return new WP_Error(self::ERROR_GENERIC, $jsonResult->error);
        }

        curl_close($curl);
        return true;
    }

    public function send_chunk($messages) {

        $logger = $this->get_logger();
        $mh = curl_multi_init();

        foreach ($messages as $message) {
            $ch = $this->build_curl($message);
            $message->ch = $ch;
            curl_multi_add_handle($mh, $ch);
        }
        $active = 1;
        $start = time();
        while ($active) {
            curl_multi_exec($mh, $active);
            curl_multi_select($mh);
            if (time() - $start > 300) {
                break;
            }
        }

        $undelivered = 0;
        foreach ($messages as $message) {
            $curl = $message->ch;
            $logger->debug(curl_errno($curl));

            if (curl_errno($curl) != 0) {
                $message->error = curl_error($curl);
                $undelivered++;
                $logger->error('cURL error: ' . curl_errno($curl) . ' - ' . curl_error($curl));
            } else {

                $code = curl_getinfo($curl, CURLINFO_HTTP_CODE);

                if ($code < 200 || $code > 299) {
                    $message->error = 'HTTP Error code ' . $code . ' ' . curl_multi_getcontent($curl);
                    $undelivered++;
                    $logger->error('Http error ' . $code . ' ' . curl_multi_getcontent($curl));
                } else {
                    /*
                    $jsonResult = json_decode($response);

                    if ($jsonResult === false) {
                        $undelivered++;
                        $message->error = 'Unable to decode the response: ' . $response;
                    } else {
                        if ($jsonResult->success === false) {
                            $undelivered++;
                            $message->error = $jsonResult->error;
                        }
                    }
                     */
                }
            }
        }

        curl_multi_close($mh);

        if ($undelivered) {
            return new WP_Error(self::ERROR_GENERIC, 'Unable to deliver all messages');
        }
        return true;
    }

}
